@@ -4,6 +4,7 @@ import glob
 import os
 import tqdm
 from collections import defaultdict
+from lxml import etree as ET
 
 
 from acdh_tei_pyutils.tei import TeiEnricher
@@ -82,3 +83,50 @@ def mentions_to_indices(files, indices, event_title):  # pragma: no cover
             except IndexError:
                 pass
         doc.tree_to_file(file=x)
+    
+    all_ent_nodes = {}
+    for x in index_files:
+        doc = TeiEnricher(x)
+        ent_nodes = doc.any_xpath('.//tei:body//*[@xml:id]')
+        for ent in ent_nodes:
+            all_ent_nodes[ent.xpath('@xml:id')[0]] = ent
+    
+    click.echo(
+        click.style(
+            f"writing {len(all_ent_nodes)} index entries into {len(files)} files",
+            fg='green'
+        )
+    )
+    for x in tqdm.tqdm(files):
+        filename = os.path.split(x)[1]
+        doc = TeiEnricher(x)
+        root_node = doc.tree.getroot()
+        refs = doc.any_xpath('.//tei:rs[@ref]/@ref')
+        ent_dict = defaultdict(list)
+        for ref in refs:
+            ent_id = ref[1:]
+            try:
+                index_ent = all_ent_nodes[ent_id]
+                ent_dict[index_ent.tag].append(index_ent)
+            except KeyError:
+                continue
+        back_node = ET.Element("{http://www.tei-c.org/ns/1.0}back")
+        for key in ent_dict.keys():
+            if key.endswith('person'):
+                list_person = ET.Element("{http://www.tei-c.org/ns/1.0}listPerson")
+                back_node.append(list_person)
+                for ent in ent_dict[key]:
+                    list_person.append(ent)
+            if key.endswith('place'):
+                list_place = ET.Element("{http://www.tei-c.org/ns/1.0}listPlace")
+                back_node.append(list_person)
+                for ent in ent_dict[key]:
+                    list_person.append(ent)
+        root_node.append(back_node)
+        doc.tree_to_file(file=x)
+    click.echo(
+        click.style(
+            f"DONE",
+            fg='green'
+        )
+    )
